"""Wisdom module."""
import json
import random
from dataclasses import asdict, dataclass
from pathlib import Path

from marshmallow_dataclass import class_schema

QUOTES_DB = Path(__file__).parent / "data" / "quotes.json"


@dataclass
class QuoteSchema:
    """Quote schema."""

    season: int
    episode: int
    title: str
    description: str

    def __iter__(self):
        """Iterator to return a dict of the quote."""
        for key in asdict(self):
            yield key, getattr(self, key)

    def __str__(self) -> str:
        """Return a pretty quote."""
        return f"{self.season}x{self.episode} - {self.title}\n{self.description}"


@dataclass
class QuoteList:
    """List of QuoteSchema."""

    data: list[QuoteSchema]


class Quotes:
    """Load quotes from a json db file."""

    def __init__(self, filename: Path = QUOTES_DB):
        """Initialize the Quotes class."""
        self.filename = filename
        self.quotes = self.load()

    def load(self) -> QuoteList:
        """Load quotes from a json file."""
        with open(self.filename, encoding="utf-8") as file:
            data = json.load(file)
            schema = class_schema(QuoteList)()
            return schema.load(data)

    def dump(self, quotes: QuoteList, filename: Path = None) -> None:
        """Dump quotes to a json file."""
        if not filename:
            filename = self.filename

        schema = class_schema(QuoteList)()
        data = schema.dump(quotes)
        with open(filename, "w", encoding="utf-8") as quotes_file:
            json.dump(data, quotes_file, indent=4)

    def select_quote(self, season: int, episode: int) -> QuoteSchema:
        """Return a quote from a specific season and episode."""
        if season and episode:
            quote = next(
                (quote for quote in self.quotes.data if quote.season == season and quote.episode == episode),
                None,
            )
        elif season:
            quote = next(
                (quote for quote in self.quotes.data if quote.season == season),
                None,
            )
        elif episode:
            quote = next(
                (quote for quote in self.quotes.data if quote.episode == episode),
                None,
            )
        return quote

    def random_quote(self) -> QuoteSchema:
        """Return a random quote."""
        quote = random.choice(self.quotes.data)  # nosec
        return quote


QUOTESCHEMA = class_schema(QuoteSchema)()
QUOTELISTSCHEMA = class_schema(QuoteList)()



"""Argparse configuration for wise_yoda."""
import argparse
import json
from pathlib import Path

from .wisdom import Quotes


def main():
    """Parse arguments and run the program."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-f", "--filename", help="File to read quotes from", default=Path().cwd() / "data" / "quotes.json"
    )

    # add option to select quote by season, episode, or random
    parser.add_argument(
        "-s",
        "--season",
        help="Select a quote by season",
        type=int,
        default=None,
    )
    parser.add_argument(
        "-e",
        "--episode",
        help="Select a quote by episode",
        type=int,
        default=None,
    )
    parser.add_argument(
        "-r",
        "--random",
        help="Select a random quote",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "-j",
        "--json",
        help="Output quote as json",
        action="store_true",
        default=False,
    )
    args = parser.parse_args()

    # Demo of the Quote class
    if args.random or (not args.season and not args.episode):
        quote = Quotes().random_quote()
    else:
        quote = Quotes().select_quote(args.season, args.episode)

    if args.json:
        print(json.dumps(dict(quote), indent=2))
    else:
        print(quote)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wise_yoda']

package_data = \
{'': ['*'], 'wise_yoda': ['data/*']}

install_requires = \
['marshmallow-dataclass>=8.5.10,<9.0.0']

entry_points = \
{'console_scripts': ['wiseyoda = wise_yoda.cli:main']}

setup_kwargs = {
    'name': 'wiseyoda',
    'version': '1.0.1',
    'description': 'Quote the wise and powerful master Yoda.',
    'long_description': '#\n\n<div align="center">\n  <img width="1072" alt="logo" src="https://user-images.githubusercontent.com/1636709/210475936-9943ee5d-6bec-488d-a309-7a0df2312291.png">\n  <h1>WiseYoda</h1>\n\n  <p>\n    Quotes from the <a href="https://github.com/Mikaayenson/WiseYoda">Wise Yoda</a>\n  </p>\n\n\n<!-- Badges -->\n\n[![Supported Python versions](https://img.shields.io/badge/python-3.8+-blue.svg)](https://www.python.org/downloads/)\n[![Python Testing](https://github.com/Mikaayenson/WiseYoda/actions/workflows/python-testing.yml/badge.svg)](https://github.com/Mikaayenson/WiseYoda/actions/workflows/python-testing.yml)\n\n<h5>\n    <a href="https://github.com/Mikaayenson/WiseYoda/issues/">Report Bug</a>\n  <span> · </span>\n    <a href="https://github.com/Mikaayenson/WiseYoda/issues/">Request Feature</a>\n  </h5>\n</div>\n\n<br />\n\n<!-- About the Project -->\n## :star2: About the Project\n\nSimple library to obtain wisdom from the wise Master Yoda in the form of quotes. Create a [feature request](https://github.com/Mikaayenson/WiseYoda/issues) if there are some useful features/commands that you hate to remember!\n\n<!-- Screenshots -->\n### :camera: Screenshots\n\n<p align="center">\n<img width="710" alt="Help" src="https://user-images.githubusercontent.com/1636709/210474079-0fd9c801-d6ba-4c87-8244-41a4c4ba5ed1.png">\n<img width="710" alt="Simple" src="https://user-images.githubusercontent.com/1636709/210474075-5ad35761-5765-49e7-ae22-62ff4aec85e1.png">\n<img width="710" alt="Complex" src="https://user-images.githubusercontent.com/1636709/210474078-d8c3c528-9bc6-4934-a970-4d372e3202a1.png">\n\n</p>\n\n<!-- Getting Started -->\n## :toolbox: Getting Started\n\n<!-- Prerequisites -->\n### :bangbang: Prerequisites\n\nThis project uses poetry as the python package manager\n\n- `poetry` Follow the [poetry install](https://python-poetry.org/docs/) guide\n- `python3.8+` Download from [python](https://www.python.org/downloads/) (ideally `3.10`)\n\n```bash\n   pip install wiseyoda\n```\n\n<!-- Usage -->\n## :eyes: Usage\n\nReminder: These commands must be run in the virtualenv where you installed the dependencies.\n\n```bash\n  from wise_yoda import Quotes\n  lesson = Quotes().random_quote()\n  lesson = Quotes().select_quote(season=1, episode=1)\n```\n\n<!-- Run Locally -->\n### :running: Run Locally\n\nClone the project\n\n```bash\n  git clone https://github.com/Mikaayenson/WiseYoda.git\n```\n\nGo to the project directory\n\n```bash\n  cd wise_yoda\n```\n\nInstall system and Python dependencies\n\n```bash\n  make deps\n```\n\nRun wiseyoda\n\n```bash\n  wiseyoda --help\n```\n\n<!-- Development -->\n### :construction: Development\n\n\n<div align="center">\n  <img width="710" alt="makefile" src="https://user-images.githubusercontent.com/1636709/210474182-474a778f-9267-4e7c-84d3-3edb48cc9f8a.png">\n</div>\n\nInstall pre-commit\n\n```bash\n  pre-commit install\n```\n\nUpdate Python dependencies\n\n```bash\n  make deps-py-update\n```\n\n<!-- Running Tests -->\n### :test_tube: Running Tests\n\nRun tests\n\n```bash\n  make test\n```\n\nRun linter\n\n```bash\n  make check\n```\n\n<!-- Build: Poetry -->\n### :triangular_flag_on_post: Build: Python Package\n\nBuild this project as a `sdist` and `wheel`\n\n```bash\n  make build\n```\n\n\n<!-- License -->\n## :warning: License\n\nDistributed under the Apache2.0 License. See LICENSE.txt for more information.\n',
    'author': 'Mika Ayenson',
    'author_email': 'Mika.ayenson@elastic.co',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Mikaayenson/WiseYoda',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""WiseYoda package."""
from .wisdom import (  # noqa: E402,
    QUOTELISTSCHEMA,
    QUOTES_DB,
    QUOTESCHEMA,
    QuoteList,
    Quotes,
    QuoteSchema,
)

__all__ = (
    "Quotes",
    "QuoteList",
    "QuoteSchema",
    "QUOTES_DB",
    "QUOTESCHEMA",
    "QUOTELISTSCHEMA",
)
